@@ -20,7 +20,10 @@
  * perform various window managment tasks such as desktop changes, window
  * movement, etc.
  *
+ * For examples on libxdo usage, the xdotool source code is a good reference.
+ *
  * @see xdo.h
+ * @see xdo_new
  */
 
 /**
@@ -31,14 +34,16 @@
  * instead of pixels.
  */
 #define SIZE_USEHINTS (1L << 0)
+#define SIZE_USEHINTS_X (1L << 1)
+#define SIZE_USEHINTS_Y (1L << 2)
 
 /**
  * CURRENTWINDOW is a special identify for xdo input faking (mouse and
  * keyboard) functions like xdo_keysequence that indicate we should target the
  * current window, not a specific window.
  *
- * Generally, this means we will use XTEST instead of XSendEvent when
- * sending events.
+ * Generally, this means we will use XTEST instead of XSendEvent when sending
+ * events.
  */
 #define CURRENTWINDOW (0)
 
@@ -75,7 +80,7 @@ typedef struct xdo {
   /** The Display for Xlib */
   Display *xdpy;
 
-  /** The display name */
+  /** The display name, if any. NULL if not specified. */
   char *display_name;
 
   /** @internal Array of known keys/characters */
@@ -119,7 +124,7 @@ typedef struct xdo_active_mods {
 
 
 /**
- * Search only window title.
+ * Search only window title. DEPRECATED - Use SEARCH_NAME
  * @see xdo_window_search
  */
 #define SEARCH_TITLE (1UL << 0)
@@ -137,13 +142,13 @@ typedef struct xdo_active_mods {
 #define SEARCH_NAME (1UL << 2)
 
 /**
- * Search only window class.
+ * Search only window pid.
  * @see xdo_window_search
  */
 #define SEARCH_PID  (1UL << 3)
 
 /**
- * Search only window class.
+ * Search only visible windows.
  * @see xdo_window_search
  */
 #define SEARCH_ONLYVISIBLE  (1UL << 4)
@@ -168,10 +173,10 @@ typedef struct xdo_active_mods {
  * @see xdo_window_search
  */
 typedef struct xdo_search {
-  char *title;        /** pattern to test against a window title */
-  char *winclass;     /** pattern to test against a window class */
-  char *winclassname; /** pattern to test against a window class */
-  char *winname;      /** pattern to test against a window name */
+  const char *title;        /** pattern to test against a window title */
+  const char *winclass;     /** pattern to test against a window class */
+  const char *winclassname; /** pattern to test against a window class */
+  const char *winname;      /** pattern to test against a window name */
   int pid;            /** window pid (From window atom _NET_WM_PID) */
   long max_depth;     /** depth of search. 1 means only toplevel windows */
   int only_visible;   /** boolean; set true to search only visible windows */
@@ -183,7 +188,7 @@ typedef struct xdo_search {
   enum { SEARCH_ANY, SEARCH_ALL } require;
   
   /** bitmask of things you are searching for, such as SEARCH_NAME, etc.
-   * @see SEARCH_NAME
+   * @see SEARCH_NAME, SEARCH_CLASS, SEARCH_PID, SEARCH_CLASSNAME, etc
    */
   unsigned int searchmask; 
 } xdo_search_t;
@@ -195,7 +200,7 @@ typedef struct xdo_search {
  * Create a new xdo_t instance.
  *
  * @param display the string display name, such as ":0". If null, uses the
- * environment variable DISPLAY.
+ * environment variable DISPLAY just like XOpenDisplay(NULL).
  *
  * @return Pointer to a new xdo_t or NULL on failure
  */
@@ -224,7 +229,6 @@ const char *xdo_version(void);
  */
 void xdo_free(xdo_t *xdo);
 
-
 /**
  * Move the mouse to a specific location.
  *
@@ -272,7 +276,7 @@ int xdo_mousedown(const xdo_t *xdo, Window window, int button);
 int xdo_mouseup(const xdo_t *xdo, Window window, int button);
 
 /**
- * Get the current mouse location
+ * Get the current mouse location (coordinates and screen number).
  *
  * @param x integer pointer where the X coordinate will be stored
  * @param y integer pointer where the Y coordinate will be stored
@@ -280,6 +284,28 @@ int xdo_mouseup(const xdo_t *xdo, Window window, int button);
  */
 int xdo_mouselocation(const xdo_t *xdo, int *x, int *y, int *screen_num);
 
+/**
+ * Get the window the mouse is currently over
+ *
+ * @param window_ret Winter pointer where the window will be stored.
+ */
+int xdo_mousewindow(const xdo_t *xdo, Window *window_ret);
+
+/**
+ * Get all mouse location-related data.
+ *
+ * If null is passed for any parameter, we simply do not store it.
+ * Useful if you only want the 'y' coordinate, for example.
+ *
+ * @param x integer pointer where the X coordinate will be stored
+ * @param y integer pointer where the Y coordinate will be stored
+ * @param screen_num integer pointer where the screen number will be stored
+ * @param window Window pointer where the window/client the mouse is over
+ *   will be stored.
+ */
+int xdo_mouselocation2(const xdo_t *xdo, int *x_ret, int *y_ret,
+                       int *screen_num_ret, Window *window_ret);
+
 /**
  * Wait for the mouse to move from a location. This function will block
  * until the condition has been satisified.
@@ -307,11 +333,22 @@ int xdo_mouse_wait_for_move_to(const xdo_t *xdo, int dest_x, int dest_y);
  */
 int xdo_click(const xdo_t *xdo, Window window, int button);
 
+/**
+ * Send a one or more clicks for a specific mouse button at the current mouse
+ * location.
+ *
+ * @param window The window you want to send the event to or CURRENTWINDOW
+ * @param button The mouse button. Generally, 1 is left, 2 is middle, 3 is
+ *    right, 4 is wheel up, 5 is wheel down.
+ */
+int xdo_click_multiple(const xdo_t *xdo, Window window, int button,
+                       int repeat, useconds_t delay);
+
 /**
  * Type a string to the specified window.
  *
  * If you want to send a specific key or key sequence, such as "alt+l", you
- * want instead xdo_keysequence.
+ * want instead xdo_keysequence(...).
  *
  * @param window The window you want to send keystrokes to or CURRENTWINDOW
  * @param string The string to type, like "Hello world!"
@@ -459,11 +496,23 @@ int xdo_window_setprop (const xdo_t *xdo, Window wid, const char *property,
  * Change the window's classname and or class.
  *
  * @param name The new class name. If NULL, no change.
- * @paramclass The new class. If NULL, no change.
+ * @param class The new class. If NULL, no change.
  */
 int xdo_window_setclass(const xdo_t *xdo, Window wid, const char *name,
                         const char *class);
 
+/**
+ * Set the override_redirect value for a window. This generally means
+ * whether or not a window manager will manage this window.
+ *
+ * If you set it to 1, the window manager will usually not draw borders on the
+ * window, etc. If you set it to 0, the window manager will see it like a
+ * normal application window.
+ *
+ */
+int xdo_window_set_override_redirect(const xdo_t *xdo, Window wid,
+                                     int override_redirect);
+
 /**
  * Focus a window.
  *
@@ -555,6 +604,19 @@ int xdo_window_map(const xdo_t *xdo, Window wid);
  */
 int xdo_window_unmap(const xdo_t *xdo, Window wid);
 
+/**
+ * Minimize a window.
+ */
+int xdo_window_minimize(const xdo_t *xdo, Window wid);
+
+/** 
+ * Reparents a window
+ *
+ * @param wid_source the window to reparent
+ * @param wid_target the new parent window
+ */
+int xdo_window_reparent(const xdo_t *xdo, Window wid_source, Window wid_target);
+
 /**
  * Get a window's location.
  *
@@ -590,6 +652,14 @@ int xdo_get_window_size(const xdo_t *xdo, Window wid, unsigned int *width_ret,
  */
 int xdo_window_get_active(const xdo_t *xdo, Window *window_ret);
 
+/**
+ * Get a window ID by clicking on it. This function blocks until a selection
+ * is made.
+ *
+ * @param window_ret Pointer to Window where the selected window is stored.
+ */
+int xdo_window_select_with_click(const xdo_t *xdo, Window *window_ret);
+
 /**
  * Set the number of desktops.
  * Uses _NET_NUMBER_OF_DESKTOPS of the EWMH spec.
@@ -719,4 +789,52 @@ int xdo_set_active_modifiers(const xdo_t *xdo, Window window,
  */
 void xdo_free_active_modifiers(xdo_active_mods_t *active_mods);
 
+/**
+ * Get the position of the current viewport.
+ *
+ * This is only relevant if your window manager supports
+ * _NET_DESKTOP_VIEWPORT 
+ */
+int xdo_get_desktop_viewport(const xdo_t *xdo, int *x_ret, int *y_ret);
+
+/**
+ * Set the position of the current viewport.
+ *
+ * This is only relevant if your window manager supports
+ * _NET_DESKTOP_VIEWPORT
+ */
+int xdo_set_desktop_viewport(const xdo_t *xdo, int x, int y);
+
+/**
+ * Kill a window and the client owning it.
+ *
+ */
+int xdo_window_kill(const xdo_t *xdo, Window window);
+
+/**
+ * Find a client window that is a parent of the window given
+ */
+#define XDO_FIND_PARENTS (0)
+
+/**
+ * Find a client window that is a child of the window given
+ */
+#define XDO_FIND_CHILDREN (1)
+
+/**
+ * Find a client window (child) in a given window. Useful if you get the
+ * window manager's decorator window rather than the client window.
+ */
+int xdo_window_find_client(const xdo_t *xdo, Window window, Window *window_ret,
+                           int direction);
+
+/**
+ * Get a window's name, if any.
+ *
+ * TODO(sissel): Document
+ */
+int xdo_get_window_name(const xdo_t *xdo, Window window, 
+                        unsigned char **name_ret, int *name_len_ret,
+                        int *name_type);
+
 #endif /* ifndef _XDO_H_ */
